<?php
class WC_Gateway_Linepay extends WC_Payment_Gateway {

		public static function instance() {
			new self();
		}

		/**
		 * Constructor for the gateway.
		 */
		public function __construct() {
			$this->id                 = 'linepay_gateway';
			$this->icon               = apply_filters('woocommerce_offline_icon', '');
			$this->has_fields         = false;
			$this->method_title       = __( 'LINE Pay付款', 'wc-gateway-linepay' );
			$this->method_description = __( '讓使用者使用LINE付款', 'wc-gateway-linepay' );
		  
			// Load the settings.
			$this->init_form_fields();
			$this->init_settings();
		  
			// Define user set variables
			$this->title        = $this->get_option( 'title' );
			$this->description  = $this->get_option( 'description' );
			$this->instructions = $this->get_option( 'instructions', $this->description );
		  
			// Actions
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'thankyou_page' ) , 10 ,1);
		  
			// Customer Emails
			add_action( 'woocommerce_email_before_order_table', array( $this, 'email_instructions' ), 10, 3 );
			if(strpos($_SERVER['REQUEST_URI'], '/linepay/confirm')===0||strpos($_SERVER['REQUEST_URI'], '/linepay/confirm')>0){
				$this->LinePay_Confirm();
			}
		}
	
		/**
		 * Initialize Gateway Settings Form Fields
		 */
		public function init_form_fields() {
	  
			$this->form_fields = apply_filters( 'wc_linepay_form_fields', array(
		  
				'enabled' => array(
					'title'   => __( '啟用', 'wc-gateway-linepay' ),
					'type'    => 'checkbox',
					'label'   => __( '啟用LINE Pay付款', 'wc-gateway-linepay' ),
					'default' => 'yes'
				),
				
				'title' => array(
					'title'       => __( '顯示名稱', 'wc-gateway-linepay' ),
					'type'        => 'text',
					'description' => __( 'LINE Pay付款', 'wc-gateway-linepay' ),
					'default'     => __( 'LINE Pay付款', 'wc-gateway-linepay' ),
					'desc_tip'    => true,
				),
			) );
		}
	
	
		/**
		 * Output for the order received page.
		 */
		public function thankyou_page($order_id) {
			global $wpdb,$woocommerce;
			$order = wc_get_order($order_id);
			$sql="SELECT `order_code`  FROM `{$wpdb->prefix}linepay_order` WHERE `order_id` = '".$order_id."'";
			$code=$wpdb->get_var($sql);
			if(!empty($code)&&$code=='0000'){
				 $order->update_status( 'wc-processing' , __( 'LINE Pay付款', 'wc-gateway-linepay' ) );
			}
			// Remove cart
			WC()->cart->empty_cart();
			if ( $this->instructions ) {
				echo wpautop( wptexturize( $this->instructions ) );
			}
		}
	
	
		/**
		 * Add content to the WC emails.
		 *
		 * @access public
		 * @param WC_Order $order
		 * @param bool $sent_to_admin
		 * @param bool $plain_text
		 */
		public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
		
			if ( $this->instructions && ! $sent_to_admin && $this->id === $order->payment_method && $order->has_status( 'on-hold' ) ) {
				echo wpautop( wptexturize( $this->instructions ) ) . PHP_EOL;
			}
		}
	
	
		/**
		 * Process the payment and return the result
		 *
		 * @param int $order_id
		 * @return array
		 */
		public function process_payment( $order_id ) {
			global $wpdb;
			session_start();
			$channelId = get_option('channelId');
			$channelSecret = get_option('channelSecret');
			if(get_option( 'testmode' )==1){
				$sandBox = 'https://sandbox-api-pay.line.me';
			}
			else{
				$sandBox = 'https://api-pay.line.me';
			}
			
			$uri = '/v3/payments/request';
			$Nonce = date('c') . uniqid('-');


			$order = wc_get_order( $order_id );
			
			// Mark as on-hold (we're awaiting the payment)
			$order->update_status( 'pending', __( 'LINE Pay付款', 'wc-gateway-linepay' ) );
			
			// Reduce stock levels
			wc_reduce_stock_levels($order_id);
			
			$order_items = $order->get_items();

			$products = array();

			$packageAmount = 0;
			foreach ($order_items as $item_id => $item) {
				$product = $item->get_product();

				$active_price   = $product->get_price(); 

			    $product_name   = $item->get_name(); 

				$item_quantity = $item->get_quantity();

				$product_image_id = $product->get_gallery_image_ids()[0];
				$image_url = wp_get_attachment_url( $product_image_id );

				$packageAmount += $active_price * $item_quantity;
				$sellproduct = array(
			  		'id' => $item_id,
			  		'name' =>  $product_name,
			  		'imageUrl' => $image_url,
			  		'quantity' => $item_quantity,
			  		'price' => $active_price,
			  	);
				array_push($products,$sellproduct);
			}

			$sellproduct = array(
		  		'id' => 'shipfee',
		  		'name' =>  '運送費',
		  		'quantity' => 1,
		  		'price' => $order->get_shipping_total(),
		  	);
		  	array_push($products,$sellproduct);
		  	$packageAmount +=  $order->get_shipping_total();
		  	//運費

		  	//折扣費(如果有)
		  	$total = ceil($order->get_total());

			$discount=$packageAmount - $total;

			$sellproduct = array(
		  		'id' => 'coupon',
		  		'name' =>  '折扣',
		  		'quantity' => 1,
		  		'price' => -$discount,
		  	);
	  		array_push($products,$sellproduct);

			$_SESSION["linepayamount"] = $total;

		  	$package = array(
		  		'id' => $order_id,
		  		'name' => 'package1',
		  		'products' => $products,
		  		'amount' => $total,
		  	);


		  	$packages = array();
		  	array_push($packages,$package);

		  	$postData =  [
			    'amount' => $total,
			    'currency' => 'TWD',
			    'orderId' => $order_id,
			    'packages' => $packages,
			    'redirectUrls' => [
			        'confirmUrl' => LinePay_URL."linepay/confirm",
			        'cancelUrl' => LinePay_URL."linepay/confirm",
			    ],
			    'options' => [
			    	'shipping' => [
			    		'feeAmount' => $order->get_shipping_total()
			    	]
			    ]
			];

		  	$authMacText = $channelSecret . $uri . json_encode($postData) . $Nonce;
			$Authorization = base64_encode(hash_hmac('sha256', $authMacText, $channelSecret, true));

		  	$header = array(
				'Content-Type: application/json; charset=UTF-8',
				'X-LINE-ChannelId: '.$channelId,
		        'X-LINE-Authorization-Nonce: '.$Nonce,
		        'X-LINE-Authorization: '.$Authorization
			);


			$ch = curl_init($sandBox.$uri);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postData));
			$return=curl_exec($ch);
			$result = json_decode($return, true);
			

			//紀錄訂單感謝頁面
			$sql="INSERT INTO `{$wpdb->prefix}linepay_redirect`(`order_id`, `url`) VALUES ('".$order_id."','".$this->get_return_url( $order )."')";
			$wpdb->query($sql);
			curl_close($ch);

			// Return thankyou redirect
			return array(
				'result' 	=> 'success',
				'redirect'	=> $result['info']['paymentUrl']['web']
			);
		}

		public function LinePay_Confirm(){
			global $wpdb;
			session_start();
			$transactionId = $_GET['transactionId'];
			$channelId = get_option('channelId');
			$channelSecret = get_option('channelSecret');

			if(get_option( 'testmode' )==1){
				$sandBox = 'https://sandbox-api-pay.line.me';
			}
			else{
				$sandBox = 'https://api-pay.line.me';
			}

			$uri = '/v3/payments/'.$transactionId.'/confirm';
			$Nonce = date('c') . uniqid('-');

			$postData = array(
		    	'amount' => $_SESSION["linepayamount"]
		    	, 'currency' => "TWD"
		  	);

			$authMacText = $channelSecret . $uri . json_encode($postData) . $Nonce;
			$Authorization = base64_encode(hash_hmac('sha256', $authMacText, $channelSecret, true));

			$header = array(
				'Content-Type: application/json; charset=UTF-8',
				'X-LINE-ChannelId: '.$channelId,
		        'X-LINE-Authorization-Nonce: '.$Nonce,
		        'X-LINE-Authorization: '.$Authorization
			);

			$ch = curl_init($sandBox.$uri);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postData));
			$result = json_decode(curl_exec($ch), true);
			curl_close($ch);
			$returnCode=$result["returnCode"];
			
			$orderId=$result["info"]["orderId"];
			$sql = "INSERT INTO `{$wpdb->prefix}linepay_order`(`order_id`, `order_code`, `order_desc`,`transaction_id`) VALUES ('".$result["info"]["orderId"]."','".$result['returnCode']."','".$result['returnMessage']."','".$result["info"]["transactionId"]."')";
			$wpdb->query($sql);

			if($returnCode=="0000"){
				$sql="SELECT url FROM `{$wpdb->prefix}linepay_redirect` WHERE order_id ='".$orderId."'";
				$url = $wpdb->get_var($sql);
				wp_redirect( $url );
			}
			else{
				echo "似乎發生了點問題,請回報給我們知道";
			}
			exit();
		}
	
  }
 ?>
